@@ -25,12 +25,13 @@ from .import_utils import (RESERVED_SYMBOLS,
                            _get_str_symbol_identifiers,
                            _parse_special_functions,
                            generate_measurement_symbol,
+                           generate_regularization_symbol,
                            noise_distribution_to_cost_function,
                            noise_distribution_to_observable_transformation,
                            smart_subs, smart_subs_dict, toposort_symbols)
 from .logging import get_logger, log_execution_time, set_log_level
 from .ode_export import (
-    ODEExporter, ODEModel, symbol_with_assumptions
+    ODEExporter, ODEModel, symbol_with_assumptions, _default_simplify
 )
 
 
@@ -210,16 +211,19 @@ class SbmlImporter:
             model_name: str,
             output_dir: Union[str, Path] = None,
             observables: Dict[str, Dict[str, str]] = None,
+            event_observables: Dict[str, Dict[str, str]] = None,
             constant_parameters: Iterable[str] = None,
             sigmas: Dict[str, Union[str, float]] = None,
+            event_sigmas: Dict[str, Union[str, float]] = None,
             noise_distributions: Dict[str, Union[str, Callable]] = None,
+            event_noise_distributions: Dict[str, Union[str, Callable]] = None,
             verbose: Union[int, bool] = logging.ERROR,
             assume_pow_positivity: bool = False,
             compiler: str = None,
             allow_reinit_fixpar_initcond: bool = True,
             compile: bool = True,
             compute_conservation_laws: bool = True,
-            simplify: Callable = lambda x: sp.powsimp(x, deep=True),
+            simplify: Optional[Callable] = _default_simplify,
             cache_simplify: bool = False,
             log_as_log10: bool = True,
             generate_sensitivity_code: bool = True,
@@ -249,18 +253,33 @@ class SbmlImporter:
             dictionary( observableId:{'name':observableName
             (optional), 'formula':formulaString)}) to be added to the model
 
+        :param event_observables:
+            dictionary( eventObservableId:{'name':eventObservableName
+            (optional), 'event':eventId, 'formula':formulaString)}) to be
+            added to the model
+
         :param constant_parameters:
             list of SBML Ids identifying constant parameters
 
         :param sigmas:
             dictionary(observableId: sigma value or (existing) parameter name)
 
+        :param event_sigmas:
+            dictionary(eventObservableId: sigma value or (existing) parameter
+            name)
+
         :param noise_distributions:
             dictionary(observableId: noise type).
             If nothing is passed for some observable id, a normal model is
             assumed as default. Either pass a noise type identifier, or a
             callable generating a custom noise string.
 
+        :param event_noise_distributions:
+            dictionary(eventObservableId: noise type).
+            If nothing is passed for some observable id, a normal model is
+            assumed as default. Either pass a noise type identifier, or a
+            callable generating a custom noise string.
+
         :param verbose:
             verbosity level for logging, ``True``/``False`` default to
             ``logging.Error``/``logging.DEBUG``
@@ -322,9 +341,15 @@ class SbmlImporter:
         if sigmas is None:
             sigmas = {}
 
+        if event_sigmas is None:
+            event_sigmas = {}
+
         if noise_distributions is None:
             noise_distributions = {}
 
+        if event_noise_distributions is None:
+            event_noise_distributions = {}
+
         self._reset_symbols()
         self.sbml_parser_settings.setParseLog(
             sbml.L3P_PARSE_LOG_AS_LOG10 if log_as_log10 else
@@ -339,7 +364,16 @@ class SbmlImporter:
                 )
             compute_conservation_laws = False
 
-        self._process_observables(observables, sigmas, noise_distributions)
+        self._process_observables(
+            observables,
+            sigmas,
+            noise_distributions
+        )
+        self._process_event_observables(
+            event_observables,
+            event_sigmas,
+            event_noise_distributions
+        )
         self._replace_compartments_with_volumes()
 
         self._clean_reserved_symbols()
@@ -630,7 +664,7 @@ class SbmlImporter:
                 self.sbml.getConversionFactor()
             )
         else:
-            conversion_factor = 1.0
+            conversion_factor = 1
 
         for s in self.sbml.getListOfSpecies():
             if self.is_assignment_rule_target(s):
@@ -871,8 +905,8 @@ class SbmlImporter:
         ]
 
         for reaction_index, reaction in enumerate(reactions):
-            for element_list, sign in [(reaction.getListOfReactants(), -1.0),
-                                       (reaction.getListOfProducts(), 1.0)]:
+            for element_list, sign in [(reaction.getListOfReactants(), -1),
+                                       (reaction.getListOfProducts(), 1)]:
                 for element in element_list:
                     stoichiometry = self._get_element_stoichiometry(
                         element
@@ -893,17 +927,16 @@ class SbmlImporter:
                     self.stoichiometric_matrix[species['index'],
                                                reaction_index] += \
                         sign * stoichiometry * species['conversion_factor']
-
             if reaction.isSetId():
                 sym_math = self._local_symbols[reaction.getId()]
             else:
                 sym_math = self._sympy_from_sbml_math(reaction.getKineticLaw())
 
             self.flux_vector[reaction_index] = sym_math
-            if any([
+            if any(
                 str(symbol) in reaction_ids
                 for symbol in self.flux_vector[reaction_index].free_symbols
-            ]):
+            ):
                 raise SBMLException(
                     'Kinetic laws involving reaction ids are currently'
                     ' not supported!'
@@ -1138,7 +1171,6 @@ class SbmlImporter:
                 'name': event_id,
                 'value': trigger,
                 'state_update': sp.MutableDenseMatrix(bolus),
-                'event_observable': None,
                 'initial_value':
                     trigger_sbml.getInitialValue() if trigger_sbml is not None
                     else True,
@@ -1146,10 +1178,10 @@ class SbmlImporter:
 
     @log_execution_time('processing SBML observables', logger)
     def _process_observables(
-            self,
-            observables: Optional[Dict[str, Dict[str, str]]] = None,
-            sigmas: Optional[Dict[str, Union[str, float]]] = None,
-            noise_distributions: Optional[Dict[str, str]] = None
+        self,
+        observables: Union[Dict[str, Dict[str, str]], None],
+        sigmas: Dict[str, Union[str, float]],
+        noise_distributions: Dict[str, str]
     ) -> None:
         """
         Perform symbolic computations required for observable and objective
@@ -1169,28 +1201,8 @@ class SbmlImporter:
             See :py:func:`sbml2amici`.
         """
 
-        if sigmas is None or observables is None:
-            sigmas = {}
-        else:
-            # Ensure no non-existing observableIds have been specified
-            # (no problem here, but usually an upstream bug)
-            unknown_ids = set(sigmas.keys()) - set(observables.keys())
-            if unknown_ids:
-                raise ValueError(
-                    f"Sigma provided for unknown observableIds: "
-                    f"{unknown_ids}.")
-
-        if noise_distributions is None or observables is None:
-            noise_distributions = {}
-        else:
-            # Ensure no non-existing observableIds have been specified
-            # (no problem here, but usually an upstream bug)
-            unknown_ids = set(noise_distributions.keys()) - \
-                          set(observables.keys())
-            if unknown_ids:
-                raise ValueError(
-                    f"Noise distribution provided for unknown observableIds: "
-                    f"{unknown_ids}.")
+        _validate_observables(observables, sigmas, noise_distributions,
+                              events=False)
 
         # add user-provided observables or make all species, and compartments
         # with assignment rules, observable
@@ -1202,8 +1214,6 @@ class SbmlImporter:
             self.symbols[SymbolId.OBSERVABLE] = {
                 symbol_with_assumptions(obs): {
                     'name': definition.get('name', f'y{iobs}'),
-                    # Replace logX(.) by log(., X) since sympy cannot parse the
-                    # former.
                     'value': self._sympy_from_sbml_math(
                         definition['formula']
                     ),
@@ -1227,8 +1237,83 @@ class SbmlImporter:
         elif observables is None:
             self._generate_default_observables()
 
+        _check_symbol_nesting(self.symbols[SymbolId.OBSERVABLE],
+                              'eventObservable')
+
         self._process_log_likelihood(sigmas, noise_distributions)
 
+    @log_execution_time('processing SBML event observables', logger)
+    def _process_event_observables(
+            self,
+            event_observables: Dict[str, Dict[str, str]],
+            event_sigmas: Dict[str, Union[str, float]],
+            event_noise_distributions: Dict[str, str]
+    ) -> None:
+        """
+        Perform symbolic computations required for observable and objective
+        function evaluation.
+
+        :param event_observables:
+            See :py:func:`sbml2amici`.
+
+        :param event_sigmas:
+            See :py:func:`sbml2amici`.
+
+        :param event_noise_distributions:
+            See :py:func:`sbml2amici`.
+        """
+        if event_observables is None:
+            return
+
+        _validate_observables(event_observables, event_sigmas,
+                              event_noise_distributions,
+                              events=True)
+
+        # gather local symbols before parsing observable and sigma formulas
+        for obs, definition in event_observables.items():
+            self.add_local_symbol(obs, symbol_with_assumptions(obs))
+            # check corresponding event exists
+            if sp.Symbol(definition['event']) not in \
+                    self.symbols[SymbolId.EVENT]:
+                raise ValueError(
+                    'Could not find an event with the event identifier '
+                    f'{definition["event"]} for the event observable with name'
+                    f'{definition["name"]}.'
+                )
+
+        self.symbols[SymbolId.EVENT_OBSERVABLE] = {
+            symbol_with_assumptions(obs): {
+                'name': definition.get('name', f'z{iobs}'),
+                'value': self._sympy_from_sbml_math(
+                    definition['formula']
+                ),
+                'event': sp.Symbol(definition.get('event')),
+                'transformation':
+                    noise_distribution_to_observable_transformation(
+                        event_noise_distributions.get(obs, 'normal')
+                    )
+            }
+            for iobs, (obs, definition) in
+            enumerate(event_observables.items())
+        }
+
+        wrong_t = sp.Symbol('t')
+        for eo in self.symbols[SymbolId.EVENT_OBSERVABLE].values():
+            if eo['value'].has(wrong_t):
+                warnings.warn(f'Event observable {eo["name"]} uses `t` in '
+                              'it\'s formula which is not the time variable. '
+                              'For the time variable, please use `time` '
+                              'instead!')
+
+        # check for nesting of observables (unsupported)
+        _check_symbol_nesting(self.symbols[SymbolId.EVENT_OBSERVABLE],
+                              'eventObservable')
+
+        self._process_log_likelihood(event_sigmas, event_noise_distributions,
+                                     events=True)
+        self._process_log_likelihood(event_sigmas, event_noise_distributions,
+                                     events=True, event_reg=True)
+
     def _generate_default_observables(self):
         """
         Generate default observables from species, compartments and
@@ -1239,8 +1324,8 @@ class SbmlImporter:
                 'name': specie['name'],
                 'value': species_id
             }
-            for ix, (species_id, specie)
-            in enumerate(self.symbols[SymbolId.SPECIES].items())
+            for species_id, specie
+            in self.symbols[SymbolId.SPECIES].items()
         }
 
         for variable, formula in itt.chain(
@@ -1263,7 +1348,9 @@ class SbmlImporter:
 
     def _process_log_likelihood(self,
                                 sigmas: Dict[str, Union[str, float]],
-                                noise_distributions: Dict[str, str]):
+                                noise_distributions: Dict[str, str],
+                                events: bool = False,
+                                event_reg: bool = False):
         """
         Perform symbolic computations required for objective function
         evaluation.
@@ -1273,25 +1360,53 @@ class SbmlImporter:
 
         :param noise_distributions:
             See :py:func:`SBMLImporter._process_observables`
+
+        :param events:
+            indicates whether the passed definitions are for observables
+            (False) or for event observables (True).
+
+        :param event_reg:
+            indicates whether log-likelihoods definitons should be processed
+            for event observable regularization (Jrz). If this is activated,
+            measurements are substituted by 0 and the observable by the
+            respective regularization symbol.
         """
 
-        for obs_id, obs in self.symbols[SymbolId.OBSERVABLE].items():
+        if events:
+            if event_reg:
+                obs_symbol = SymbolId.EVENT_OBSERVABLE
+                sigma_symbol = SymbolId.SIGMAZ
+                llh_symbol = SymbolId.LLHRZ
+            else:
+                obs_symbol = SymbolId.EVENT_OBSERVABLE
+                sigma_symbol = SymbolId.SIGMAZ
+                llh_symbol = SymbolId.LLHZ
+        else:
+            assert not event_reg
+            obs_symbol = SymbolId.OBSERVABLE
+            sigma_symbol = SymbolId.SIGMAY
+            llh_symbol = SymbolId.LLHY
+
+        for obs_id, obs in self.symbols[obs_symbol].items():
             obs['measurement_symbol'] = generate_measurement_symbol(obs_id)
+            if event_reg:
+                obs['reg_symbol'] = generate_regularization_symbol(obs_id)
 
-        self.symbols[SymbolId.SIGMAY] = {
-            symbol_with_assumptions(f'sigma_{obs_id}'): {
-                'name': f'sigma_{obs["name"]}',
-                'value': self._sympy_from_sbml_math(
-                    sigmas.get(str(obs_id), '1.0')
-                )
+        if not event_reg:
+            self.symbols[sigma_symbol] = {
+                symbol_with_assumptions(f'sigma_{obs_id}'): {
+                    'name': f'sigma_{obs["name"]}',
+                    'value': self._sympy_from_sbml_math(
+                        sigmas.get(str(obs_id), '1.0')
+                    )
+                }
+                for obs_id, obs in self.symbols[obs_symbol].items()
             }
-            for obs_id, obs in self.symbols[SymbolId.OBSERVABLE].items()
-        }
 
-        self.symbols[SymbolId.LLHY] = {}
+        self.symbols[llh_symbol] = {}
         for (obs_id, obs), (sigma_id, sigma) in zip(
-                self.symbols[SymbolId.OBSERVABLE].items(),
-                self.symbols[SymbolId.SIGMAY].items()
+                self.symbols[obs_symbol].items(),
+                self.symbols[sigma_symbol].items()
         ):
             symbol = symbol_with_assumptions(f'J{obs_id}')
             dist = noise_distributions.get(str(obs_id), 'normal')
@@ -1300,7 +1415,10 @@ class SbmlImporter:
                 _get_str_symbol_identifiers(obs_id),
                 (obs_id, obs['measurement_symbol'], sigma_id)
             )))
-            self.symbols[SymbolId.LLHY][symbol] = {
+            if event_reg:
+                value = value.subs(obs['measurement_symbol'], 0.0)
+                value = value.subs(obs_id, obs['reg_symbol'])
+            self.symbols[llh_symbol][symbol] = {
                     'name': f'J{obs["name"]}',
                     'value': value,
                     'dist': dist,
@@ -1548,7 +1666,6 @@ class SbmlImporter:
         for row in kernel:
             state_idxs = [i for i, coeff in enumerate(row) if coeff]
             coefficients = [coeff for coeff in row if coeff]
-            print(state_idxs, coefficients)
             raw_cls.append((state_idxs[0], state_idxs, coefficients),)
 
         return raw_cls
@@ -1702,8 +1819,9 @@ class SbmlImporter:
                 del self.symbols[symbol][old]
 
         # replace in values
-        for symbol in [SymbolId.OBSERVABLE, SymbolId.LLHY, SymbolId.SIGMAY,
-                       SymbolId.EXPRESSION, SymbolId.EVENT]:
+        for symbol in [SymbolId.OBSERVABLE, SymbolId.LLHY, SymbolId.LLHZ,
+                       SymbolId.SIGMAY, SymbolId.SIGMAZ, SymbolId.EXPRESSION,
+                       SymbolId.EVENT, SymbolId.EVENT_OBSERVABLE]:
             if not self.symbols.get(symbol, None):
                 continue
             for element in self.symbols[symbol].values():
@@ -1840,9 +1958,11 @@ class SbmlImporter:
                 return _get_identifier_symbol(ele)
 
         if ele.isSetStoichiometry():
-            return sp.Float(ele.getStoichiometry())
+            stoichiometry: float = ele.getStoichiometry()
+            return sp.Integer(stoichiometry) if stoichiometry.is_integer() \
+                else sp.Float(stoichiometry)
 
-        return sp.Float(1.0)
+        return sp.Integer(1)
 
     def is_assignment_rule_target(self, element: sbml.SBase) -> bool:
         """
@@ -1856,10 +1976,7 @@ class SbmlImporter:
             boolean indicating truth of function name
         """
         a = self.sbml.getAssignmentRuleByVariable(element.getId())
-        if a is None or self._sympy_from_sbml_math(a) is None:
-            return False
-
-        return True
+        return a is not None and self._sympy_from_sbml_math(a) is not None
 
     def is_rate_rule_target(self, element: sbml.SBase) -> bool:
         """
@@ -1873,10 +1990,7 @@ class SbmlImporter:
             boolean indicating truth of function name
         """
         a = self.sbml.getRateRuleByVariable(element.getId())
-        if a is None or self._sympy_from_sbml_math(a) is None:
-            return False
-
-        return True
+        return a is not None and self._sympy_from_sbml_math(a) is not None
 
 
 def _check_lib_sbml_errors(sbml_doc: sbml.SBMLDocument,
@@ -1895,7 +2009,7 @@ def _check_lib_sbml_errors(sbml_doc: sbml.SBMLDocument,
     num_fatal = sbml_doc.getNumErrors(sbml.LIBSBML_SEV_FATAL)
 
     if num_warning + num_error + num_fatal:
-        for i_error in range(0, sbml_doc.getNumErrors()):
+        for i_error in range(sbml_doc.getNumErrors()):
             error = sbml_doc.getError(i_error)
             # we ignore any info messages for now
             if error.getSeverity() >= sbml.LIBSBML_SEV_ERROR \
@@ -1928,12 +2042,12 @@ def _parse_event_trigger(trigger: sp.Expr) -> sp.Expr:
         _check_unsupported_functions_sbml(root, 'sympy.Expression')
 
         # convert relational expressions into trigger functions
-        if isinstance(trigger, sp.core.relational.LessThan) or \
-                isinstance(trigger, sp.core.relational.StrictLessThan):
+        if isinstance(trigger, (sp.core.relational.LessThan,
+                                sp.core.relational.StrictLessThan)):
             # y < x or y <= x
             return -root
-        if isinstance(trigger, sp.core.relational.GreaterThan) or \
-                isinstance(trigger, sp.core.relational.StrictGreaterThan):
+        if isinstance(trigger, (sp.core.relational.GreaterThan,
+                                sp.core.relational.StrictGreaterThan)):
             # y >= x or y > x
             return root
 
@@ -2171,3 +2285,46 @@ def _parse_special_functions_sbml(sym: sp.Expr,
         return _parse_special_functions(sym, toplevel)
     except RuntimeError as err:
         raise SBMLException(str(err))
+
+
+def _validate_observables(
+    observables: Union[Dict[str, Dict[str, str]], None],
+    sigmas: Dict[str, Union[str, float]],
+    noise_distributions: Dict[str, str],
+    events: bool = False
+) -> None:
+
+    if observables is None or not observables:
+        return
+
+    # Ensure no non-existing observableIds have been specified
+    # (no problem here, but usually an upstream bug)
+    unknown_ids = set(sigmas.keys()) - set(observables.keys())
+    if unknown_ids:
+        raise ValueError(
+            f"Sigma provided for unknown "
+            f"{'eventO' if events else 'o'}bservableIds: "
+            f"{unknown_ids}.")
+
+    # Ensure no non-existing observableIds have been specified
+    # (no problem here, but usually an upstream bug)
+    unknown_ids = set(noise_distributions.keys()) - \
+        set(observables.keys())
+    if unknown_ids:
+        raise ValueError(
+            f"Noise distribution provided for unknown "
+            f"{'eventO' if events else 'o'}bservableIds: "
+            f"{unknown_ids}.")
+
+
+def _check_symbol_nesting(symbols: Dict[sp.Symbol, Dict[str, sp.Expr]],
+                          symbol_type: str):
+    observable_syms = set(symbols.keys())
+    for obs in symbols.values():
+        if any(sym in observable_syms
+               for sym in obs['value'].free_symbols):
+            raise ValueError(
+                "Nested observables are not supported, "
+                f"but {symbol_type} `{obs['name']} = {obs['value']}` "
+                "references another observable."
+            )
